                                                        {
component: kxLabelEdit
version: 0.2
project: kxlib
autor: Larionov Stanislav (sudemon@xaker.ru)

                                                        }
unit kxLabelEdit;

interface

uses
  SysUtils, Types, Classes, QGraphics, QControls,
  QForms, QDialogs, QStdCtrls, Qt,QMenus,QExtCtrls,
  kxEdit;

type
  TLabelPosition = (lpTop, lpBottom, lpLeft, lpRight, lpNone);

  TkxLabel = class(TLabel)
  constructor Create(AOwner: TComponent); override;
  end;

  TkxLabelEdit = class(TCustomControl)
  private
    ComponentCreated: Boolean;
    FGap: Integer;
    FEditBorder: Integer;
    FEditBorderColor: TColor;
    FEditBorderStyle: TPenStyle;
    FEdit: TkxEdit;
    FEditAlignment:TAlignment;

    FEditFontParent:Boolean;
    FLabelFontParent:Boolean;

    FLabel: TkxLabel;
    FLabelPosition: TLabelPosition;
    FLabelAutoSize: Boolean;
    FLabelAutoAlignment: Boolean;
    FLabelWidth: Integer;
    FLabelAlignment:TAlignment;

    FOnLabelClick: TNotifyEvent;
    FOnLabelDblClick: TNotifyEvent;
    FOnLabelMouseEnter: TNotifyEvent;
    FOnLabelMouseLeave: TNotifyEvent;

    FOnEditReturnPressed: TNotifyEvent;
    FOnEditChange: TNotifyEvent;
    FOnEditClick: TNotifyEvent;
    FOnEditDblClick: TNotifyEvent;
    FOnEditEnter: TNotifyEvent;
    FOnEditExit: TNotifyEvent;
    FOnEditMouseEnter: TNotifyEvent;
    FOnEditMouseLeave: TNotifyEvent;

    procedure SetEditFontParent(Value: Boolean);
    procedure SetLabelFontParent(Value: Boolean);

    function GetEditPopupMenu: TPopupMenu;
    procedure SetEditPopupMenu(const Value: TPopupMenu);

    function GetEditMode: TEchoMode;
    procedure SetEditMode(const Value: TEchoMode);

    function GetEditText: string;
    procedure SetEditText(const Value: String);
    function GetEditColor: TColor;
    procedure SetEditColor(const Value: TColor);
    function GetEditFont: TFont;
    procedure SetEditFont(const Value: TFont);
    function GetEditReadOnly: Boolean;
    procedure SetEditReadOnly(const Value: Boolean);
    function GetEditAlignment: TAlignment;
    procedure SetEditAlignment(Value: TAlignment);
    function GetLabelText: string;
    procedure SetLabelText(const Value: String);
    function GetLabelFont: TFont;
    procedure SetLabelFont(const Value: TFont);
    function GetLabelAlignment: TAlignment;
    procedure SetLabelAlignment(Value: TAlignment);
    procedure SetLabelAutoSize(Value: Boolean);
    procedure SetLabelAutoAlignment(Value: Boolean);
    function  GetLabelWidth: integer;
    procedure SetLabelWidth(Value: integer);
    procedure SetLabelPosition(Value: TLabelPosition);
    procedure SetPositionNone;
    procedure SetPositionBottom;
    procedure SetPositionLeft;
    procedure SetPositionRight;
    procedure SetPositionTop;
    procedure SetGap(Value: Integer);
    procedure SetEditBorder(Value: Integer);
    procedure SetEditBorderColor(Value: TColor);
    procedure SetEditBorderStyle(Value: TPenStyle);
    procedure LabelClick(Sender: TObject);
    procedure LabelDblClick(Sender: TObject);
    procedure LabelMouseEnter(Sender: TObject);
    procedure LabelMouseLeave(Sender: TObject);

    procedure EditReturnPressed(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure EditClick(Sender: TObject);
    procedure EditDblClick(Sender: TObject);
    procedure EditEnter(Sender: TObject);
    procedure EditExit(Sender: TObject);
    procedure EditMouseEnter(Sender: TObject);
    procedure EditMouseLeave(Sender: TObject);
  protected
    procedure FontChanged; override;
    procedure Paint; override;
    procedure Resize; override;
    procedure ShowingChanged; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Color;
    property TabOrder;
    property Caption: String read GetLabelText write SetLabelText;
    property Enabled;
    property Gap: Integer read FGap write SetGap;
    property EditBorder: Integer read FEditBorder write SetEditBorder;
    property EditBorderColor: TColor read FEditBorderColor write SetEditBorderColor;
    property EditBorderStyle: TPenStyle read FEditBorderStyle write SetEditBorderStyle;
    property Text: String read GetEditText write SetEditText;
    property ParentColor;
    property EditPopupMenu: TPopupMenu read GetEditPopupMenu write SetEditPopupMenu;
    property EditAlignment:TAlignment read GetEditAlignment write SetEditAlignment ;
    property EditColor: TColor read GetEditColor write SetEditColor;
    property EditFont: TFont read GetEditFont write SetEditFont;
    property EditReadOnly: Boolean read GetEditReadOnly write SetEditReadOnly;
    property EditMode: TEchoMode read GetEditMode write SetEditMode;
    property EditFontParent: Boolean read FEditFontParent write SetEditFontParent;

    property LabelFontParent: Boolean read FLabelFontParent write SetLabelFontParent;
    property LabelAlignment:TAlignment read GetLabelAlignment write SetLabelAlignment ;
    property LabelAutoSize: Boolean read FLabelAutoSize write SetLabelAutoSize;
    property LabelAutoAlignment: Boolean read FLabelAutoAlignment write SetLabelAutoAlignment;
    property LabelFont: TFont read GetLabelFont write SetLabelFont;
    property LabelPosition: TLabelPosition read FLabelPosition write SetLabelPosition;
    property LabelWidth: Integer read GetLabelWidth write SetLabelWidth;

    property OnLabelClick: TNotifyEvent read FOnLabelClick write FOnLabelClick;
    property OnLabelDblClick: TNotifyEvent read FOnLabelDblClick write FOnLabelDblClick;
    property OnLabelMouseEnter: TNotifyEvent read FOnLabelMouseEnter write FOnLabelMouseEnter;
    property OnLabelMouseLeave: TNotifyEvent read FOnLabelMouseLeave write FOnLabelMouseLeave;

    property OnEditReturnPressed: TNotifyEvent read FOnEditReturnPressed write FOnEditReturnPressed;
    property OnEditChange: TNotifyEvent read FOnEditChange write FOnEditChange;
    property OnEditClick: TNotifyEvent read FOnEditClick write FOnEditClick;
    property OnEditDblClick: TNotifyEvent read FOnEditDblClick write FOnEditDblClick;
    property OnEditEnter: TNotifyEvent read FOnEditEnter write FOnEditEnter;
    property OnEditExit: TNotifyEvent read FOnEditExit write FOnEditExit;
    property OnEditMouseEnter: TNotifyEvent read FOnEditMouseEnter write FOnEditMouseEnter;
    property OnEditMouseLeave: TNotifyEvent read FOnEditMouseLeave write FOnEditMouseLeave;

    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
    property DragMode;
    property Anchors;

  end;

procedure Register;

var
  NewLWidth,
  NewLHeight,
  NewEWidth,
  NewEHeight: Integer;
implementation


procedure Register;
begin
  RegisterComponents('kxlib',[TkxLabelEdit]);
end;


constructor TkxLabelEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FLabel:=TkxLabel.Create(Self);
  FLabel.Parent:=Self;

  FEdit:=TkxEdit.Create(Self);
  FEdit.Parent:=Self;

  FLabel.Caption:='kxLabel';
  FLabel.Layout:=tlCenter;

  FLabelFontParent:=True;
  FEditFontParent:=True;
  FEditBorder:=1;
  FEditBorderColor:=clGray;
  FEdit.Color:=clBase;

  FGap:=5;

  FLabelPosition:=lpLeft;
  FLabel.Visible:=True;
  FEdit.Visible:=True;

  FLabel.FocusControl:=FEdit;
  Width:=300;
  FLabelWidth:=100;
  FLabelAutoSize:=True;
  FLabelAutoAlignment:=True;

  FEdit.OnReturnPressed:=EditReturnPressed;
  FEdit.OnChange:=EditChange;
  FEdit.OnClick:=EditClick;
  FEdit.OnDblClick:=EditDblClick;
  FEdit.OnEnter:=EditEnter;
  FEdit.OnExit:=EditExit;
  FEdit.OnMouseEnter:=EditMouseEnter;
  FEdit.OnMouseLeave:=EditMouseLeave;

  FLabel.OnClick:= LabelClick;
  FLabel.OnDblClick:= LabelDblClick;
  FLabel.OnMouseEnter:= LabelMouseEnter;
  FLabel.OnMouseLeave:= LabelMouseLeave;
  SetLabelAutoSize(True);
  ComponentCreated:=True;

end;

constructor TkxLabel.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TkxLabelEdit.Paint;
begin
  inherited;
  Canvas.Pen.Color:=FEditBorderColor;
  Canvas.Brush.Color:=FEditBorderColor;
  Canvas.Pen.Mode:=pmCopy;
  Canvas.Pen.Style:=FEditBorderStyle;
  Canvas.Pen.Width:=FEditBorder;

  Canvas.Rectangle(FEdit.Left-FEditBorder,
  FEdit.Top-FEditBorder+FEditBorder div 2,
  FEdit.Width+FEdit.Left+FEditBorder,
  FEdit.Height+FEdit.Top+FEditBorder);
end;

procedure TkxLabelEdit.FontChanged;
begin
  inherited;
  LabelPosition:=LabelPosition;
end;

procedure TkxLabelEdit.SetGap(Value: Integer);
begin
  FGap:=Value;
  Self.Repaint;
  FontChanged;
end;

procedure TkxLabelEdit.ShowingChanged;
begin
  inherited;
  if (csDesigning in ComponentState) then
    FontChanged;
end;

procedure TkxLabelEdit.Resize;
begin
  inherited;
  if ComponentCreated then FontChanged;
end;

procedure TkxLabelEdit.SetLabelPosition(Value: TLabelPosition);
begin
  FLabelPosition:=Value;
  case FLabelPosition of
    lpBottom: SetPositionBottom;
    lpLeft: SetPositionLeft;
    lpRight: SetPositionRight;
    lpTop: SetPositionTop;
    lpNone: SetPositionNone;
  end;
  Self.Repaint;
end;

procedure TkxLabelEdit.SetPositionBottom;
begin
  if FLabelAutoAlignment then FLabel.Alignment:=taLeftJustify
  else FLabel.Alignment:=FLabelAlignment;
  FLabel.GetSize(NewLWidth,NewLHeight);
  Height:=FEdit.Height + NewLHeight + FGap + (3*FEditBorder);
  FLabel.Height:=NewLHeight;
  FLabel.Width:=Width - (2*FEditBorder);
  FLabel.Left:=FEditBorder;
  FEdit.Top:=FEditBorder;
  FLabel.Top:=FEdit.Top+FEdit.Height + FGap + FEditBorder;
  FEdit.Left:=FEditBorder;
  FEdit.Width:=Width - (2*FEditBorder) ;
end;

procedure TkxLabelEdit.SetPositionNone;
begin
  FLabel.Visible:=False;
  Height:=FEdit.Height + (2*FEditBorder);
  FEdit.Left:=FEditBorder;
  FEdit.Top:=FEditBorder;
  FEdit.Width:=Width-(2*FEditBorder);
end;

procedure TkxLabelEdit.SetPositionLeft;
begin
  FLabel.Visible:=True;
  Height:=FEdit.Height + (2*FEditBorder);
  if FLabelAutoAlignment then FLabel.Alignment:=taRightJustify
  else FLabel.Alignment:=FLabelAlignment;
  FLabel.GetSize(NewLWidth,NewLHeight);
  if FLabelAutoSize then FLabel.Width:=NewLWidth
  else FLabel.Width:=FLabelWidth;
  FLabel.Left:=FEditBorder;
  FLabel.Top:=FEditBorder;
  FLabel.Height:=Height-(2*FEditBorder);
  FEdit.Left:=Flabel.Left + FLabel.Width + FEditBorder +Fgap;
  FEdit.Top:=FEditBorder;
  FEdit.Width:=Width-FEdit.Left-(2*FEditBorder);
end;


procedure TkxLabelEdit.SetPositionRight;
begin
  FLabel.Visible:=True;
  Height:=FEdit.Height + (2*FEditBorder);
  FEdit.Left:=FEditBorder;
  FEdit.Top:=FEditBorder;
  FLabel.GetSize(NewLWidth,NewLHeight);

  if FLabelAutoSize then FLabel.Width:=NewLWidth
  else FLabel.Width:=FLabelWidth;

  FEdit.Width:=Width-FLabel.Width-(2*FEditBorder)-FGap;

  FLabel.Height:=Height-(2*FEditBorder);
  FLabel.Left:=FEdit.Left + FEdit.Width + FEditBorder + Fgap;
  FLabel.Top:=FEditBorder;
  if FLabelAutoAlignment then FLabel.Alignment:=taLeftJustify
  else FLabel.Alignment:=FLabelAlignment;
end;

procedure TkxLabelEdit.SetPositionTop;

begin
  FLabel.Visible:=True;
  if FLabelAutoAlignment then FLabel.Alignment:=taLeftJustify
  else FLabel.Alignment:=FLabelAlignment;
  FLabel.GetSize(NewLWidth,NewLHeight);
  Height:=FEdit.Height + NewLHeight + FGap + (3*FEditBorder);
  FLabel.Height:=NewLHeight;
  FLabel.Width:=Width - (2*FEditBorder);
  FLabel.Top:=FEditBorder;
  FLabel.Left:=FEditBorder;
  FEdit.Top:=FLabel.Top+FLabel.Height + FGap + FEditBorder;
  FEdit.Left:=FEditBorder;
  FEdit.Width:=Width - (2*FEditBorder) ;
end;










{Label Properties}


function TkxLabelEdit.GetLabelAlignment: TAlignment;
begin
  Result:=FLabelAlignment;
end;

procedure TkxLabelEdit.SetLabelAlignment(Value: TAlignment);
begin
  FLabel.Alignment:=Value;
  FLabelAlignment:=Value;
  FLabelAutoAlignment:=False;
end;


procedure TkxLabelEdit.SetLabelAutoAlignment(Value: Boolean);
begin
  FLabelAutoAlignment:=Value;
  FontChanged;
end;


procedure TkxLabelEdit.SetLabelAutoSize(Value: Boolean);
begin
  FLabel.AutoSize:=Value;
  FLabelAutoSize:=Value;
  FontChanged;
end;


function TkxLabelEdit.GetLabelFont: TFont;
begin
  Result:=FLabel.Font;
end;

procedure TkxLabelEdit.SetLabelFont(const Value: TFont);
begin
  FLabel.Font:=Value;
  FLabelFontParent:=False;
  FontChanged;
end;


function TkxLabelEdit.GetLabelText: String;
begin
  Result:=FLabel.Caption;
end;
procedure TkxLabelEdit.SetLabelText(const Value: String);
begin
  FLabel.Caption:=Value;
  FontChanged;
end;

Function TkxLabelEdit.GetLabelWidth: integer;
begin
  result:=FLabelWidth;
end;

procedure TkxLabelEdit.SetLabelWidth(Value: integer);
begin
  FLabelWidth:=Value;
  FontChanged;
end;

procedure TkxLabelEdit.SetLabelFontParent(Value: Boolean);
begin
  FLabelFontParent:=Value;
  FLabel.ParentFont:=Value;
  LabelPosition:=LabelPosition;
end;



{Label Events}





procedure TkxLabelEdit.LabelClick(Sender: TObject);
begin
  if Assigned(FOnLabelClick) then
    FOnLabelClick(Self);
end;

procedure TkxLabelEdit.LabelDblClick(Sender: TObject);
begin
  if Assigned(FOnLabelDblClick) then
    FOnLabelDblClick(Self);
end;

procedure TkxLabelEdit.LabelMouseEnter(Sender: TObject);
begin
  if Assigned(FOnLabelMouseEnter) then
    FOnLabelMouseEnter(Self);
end;

procedure TkxLabelEdit.LabelMouseLeave(Sender: TObject);
begin
  if Assigned(FOnLabelMouseLeave) then
    FOnLabelMouseLeave(Self);
end;





{Edit Properties}

function TkxLabelEdit.GetEditAlignment: TAlignment;
begin
  Result:=FEdit.Alignment;
end;
procedure TkxLabelEdit.SetEditAlignment(Value: TAlignment);
begin
  FEdit.Alignment:=Value;
  FEditAlignment:=Value;
end;

procedure TkxLabelEdit.SetEditBorder(Value: Integer);
begin
  if (EditBorder<=4) and (EditBorder>=0) then FEditBorder:=Value
  else FEditBorder:=1;
  Self.Repaint;
  LabelPosition:=LabelPosition;
end;

procedure TkxLabelEdit.SetEditBorderColor(Value: TColor);
begin
  FEditBorderColor:=Value;
  Self.Repaint;
  LabelPosition:=LabelPosition;
end;

procedure TkxLabelEdit.SetEditBorderStyle(Value: TPenStyle);
begin
  FEditBorderStyle:=Value;
  LabelPosition:=LabelPosition;
end;

function TkxLabelEdit.GetEditColor: TColor;
begin
  Result:=FEdit.Color;
end;
procedure TkxLabelEdit.SetEditColor(const Value: TColor);
begin
  FEdit.Color:=Value;
end;


function TkxLabelEdit.GetEditFont: TFont;
begin
  Result:=FEdit.Font;
end;
procedure TkxLabelEdit.SetEditFont(const Value: TFont);
begin
  FEdit.Font:=Value;
  FEditFontParent:=False;
  FontChanged;
end;


function TkxLabelEdit.GetEditMode: TEchoMode;
begin
  Result:=FEdit.EchoMode;
end;
procedure TkxLabelEdit.SetEditMode(const Value: TEchoMode);
begin
  FEdit.EchoMode:=Value;
end;


function TkxLabelEdit.GetEditPopupMenu: TPopupMenu;
begin
  Result:=FEdit.PopupMenu;
end;
procedure TkxLabelEdit.SetEditPopupMenu(const Value: TPopupMenu);
begin
  FEdit.PopupMenu:=Value;
end;


function TkxLabelEdit.GetEditReadOnly: Boolean;
begin
  Result:=FEdit.ReadOnly;
end;
procedure TkxLabelEdit.SetEditReadOnly(const Value: Boolean);
begin
  FEdit.ReadOnly:=Value;
end;


function TkxLabelEdit.GetEditText: String;
begin
  Result:=FEdit.Text;
end;
procedure TkxLabelEdit.SetEditText(const Value: String);
begin
  FEdit.Text:=Value;
end;

procedure TkxLabelEdit.SetEditFontParent(Value: Boolean);
begin
FEditFontParent:=Value;
FEdit.ParentFont:=Value;
LabelPosition:=LabelPosition;
end;



{Edit Events}





procedure TkxLabelEdit.EditReturnPressed(Sender: TObject);
begin
  if Assigned(FOnEditReturnPressed) then
    FOnEditReturnPressed(Self);
end;

procedure TkxLabelEdit.EditChange(Sender: TObject);
begin
  if Assigned(FOnEditChange) then
    FOnEditChange(Self);
end;

procedure TkxLabelEdit.EditClick(Sender: TObject);
begin
  if Assigned(FOnEditClick) then
    FOnEditClick(Self);
end;

procedure TkxLabelEdit.EditDblClick(Sender: TObject);
begin
  if Assigned(FOnEditDblClick) then
    FOnEditDblClick(Self);
end;

procedure TkxLabelEdit.EditEnter(Sender: TObject);
begin
  if Assigned(FOnEditEnter) then
    FOnEditEnter(Self);
end;

procedure TkxLabelEdit.EditExit(Sender: TObject);
begin
  if Assigned(FOnEditExit) then
    FOnEditExit(Self);
end;

procedure TkxLabelEdit.EditMouseEnter(Sender: TObject);
begin
  if Assigned(FOnEditMouseEnter) then
    FOnEditMouseEnter(Self);
end;

procedure TkxLabelEdit.EditMouseLeave(Sender: TObject);
begin
  if Assigned(FOnEditMouseLeave) then
    FOnEditMouseLeave(Self);
end;





end.

